/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Pod.as 5261 2009-09-11 14:12:05Z merisiec $
 * --------------------------------------------------------------------------
 */

/*
 * Container which holds PodContentBase subclasses.
 */

package org.ow2.jasmine.monitoring.eos.confeditor.view
{
    import flash.display.DisplayObject;
    import flash.display.Graphics;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.events.MouseEvent;
    import flash.events.SecurityErrorEvent;
    import flash.net.FileReference;
    import flash.net.URLRequest;
    import flash.net.URLRequestMethod;
    import flash.net.URLVariables;

    import flexlib.containers.SuperTabNavigator;
    import flexlib.controls.tabBarClasses.SuperTab;

    import mx.collections.ArrayCollection;
    import mx.containers.Canvas;
    import mx.containers.HBox;
    import mx.containers.Panel;
    import mx.controls.Alert;
    import mx.controls.Button;
    import mx.controls.ButtonBar;
    import mx.controls.TextArea;
    import mx.events.ChildExistenceChangedEvent;
    import mx.events.CloseEvent;
    import mx.events.DragEvent;
    import mx.events.ResizeEvent;

    import org.ow2.jasmine.monitoring.eos.common.utils.logger.EoSLogger;
    import org.ow2.jasmine.monitoring.eos.confeditor.control.GraphXML;
    import org.ow2.jasmine.monitoring.eos.confeditor.event.PodStateChangeEvent;

// Drag events.
    [Event(name="dragStart",type="mx.events.DragEvent")]
    [Event(name="dragComplete",type="mx.events.DragEvent")]
// Resize events.
    [Event(name="minimize",type="org.ow2.jasmine.monitoring.eos.confeditor.event.PodStateChangeEvent")]
    [Event(name="maximize",type="org.ow2.jasmine.monitoring.eos.confeditor.event.PodStateChangeEvent")]
    [Event(name="restore",type="org.ow2.jasmine.monitoring.eos.confeditor.event.PodStateChangeEvent")]
    [Event(name="close",type="org.ow2.jasmine.monitoring.eos.confeditor.event.PodStateChangeEvent")]

    public class Pod extends Panel
    {
        public static const MINIMIZED_HEIGHT:Number=22;
        public static const WINDOW_STATE_DEFAULT:Number=-1;
        public static const WINDOW_STATE_MINIMIZED:Number=0;
        public static const WINDOW_STATE_MAXIMIZED:Number=1;

        public var windowState:Number; // Corresponds to one of the WINDOW_STATE variables.
        public var index:Number; // Index within the layout.

        private var controlsHolder:HBox;

        private var minimizeButton:Button;
        private var maximizeRestoreButton:Button;
        private var closeButton:Button;

        // content of pod
        private var containerCanvas:Canvas;
        private var podButtonBar:ButtonBar;
        private var newGraphButton:Button;
        private var visualButton:Button;
        private var textualButton:Button;
        private var saveButton:Button;
        private var tabbedEditor:SuperTabNavigator;
        private var xmlTextArea:TextArea;

        private var graphAvailable:ArrayCollection=new ArrayCollection([{label: "Line Graph", value: "lineChartConst"}, {label: "Area Graph", value: "areaChartConst"}, {label: "Column Graph", value: "columnChartConst"}]);

        private var headerDivider:Sprite;

        // Variables used for dragging the pod.
        private var dragStartMouseX:Number;
        private var dragStartMouseY:Number;
        private var dragStartX:Number;
        private var dragStartY:Number;
        private var dragMaxX:Number;
        private var dragMaxY:Number;

        private var _showControls:Boolean;
        private var _showControlsChanged:Boolean;

        private var _maximize:Boolean;
        private var _maximizeChanged:Boolean;

        //for the export in xml
        private var downloadFileRef:FileReference;
        private var exportRequest:URLRequest=null;
        private static const XML_DOWNLOAD_END_POINT:String="./getxml";

        /**
         * chart icon (16 pixels)
         */
        [Embed(source='/../assets/chart16.png')]
        [Bindable]
        public var chartIcon16:Class;

        /**
         * Visual edition icon (16 pixels)
         */
        [Embed(source='/../assets/visual16.png')]
        [Bindable]
        public var visualIcon16:Class;

        /**
         * Textual edition icon (16 pixels)
         */
        [Embed(source='/../assets/textual16.png')]
        [Bindable]
        public var textualIcon16:Class;

        /**
         * Save file icon (16 pixels)
         */
        [Embed(source='/../assets/save16.png')]
        [Bindable]
        public var saveIcon16:Class;

        public function Pod()
        {
            super();
            doubleClickEnabled=true;
            setStyle("titleStyleName", "podTitle");

            windowState=WINDOW_STATE_DEFAULT;
            this.addEventListener(ResizeEvent.RESIZE, resizeHandler);
            //horizontalScrollPolicy = "off";
        }

        override protected function createChildren():void
        {
            super.createChildren();

            if (!headerDivider)
            {
                headerDivider=new Sprite();
                titleBar.addChild(headerDivider);
            }

            if (!controlsHolder)
            {
                controlsHolder=new HBox();
                controlsHolder.setStyle("paddingRight", getStyle("paddingRight"));
                controlsHolder.setStyle("horizontalAlign", "right");
                controlsHolder.setStyle("verticalAlign", "middle");
                controlsHolder.setStyle("horizontalGap", 3);
                rawChildren.addChild(controlsHolder);
            }

            if (!minimizeButton)
            {
                minimizeButton=new Button();
                minimizeButton.width=14;
                minimizeButton.height=14;
                minimizeButton.styleName="minimizeButton";
                controlsHolder.addChild(minimizeButton);
            }

            if (!maximizeRestoreButton)
            {
                maximizeRestoreButton=new Button();
                maximizeRestoreButton.width=14;
                maximizeRestoreButton.height=14;
                maximizeRestoreButton.styleName="maximizeRestoreButton";
                controlsHolder.addChild(maximizeRestoreButton);
            }

            if (!closeButton)
            {
                closeButton=new Button();
                closeButton.width=14;
                closeButton.height=14;
                closeButton.styleName="closeButton";
                controlsHolder.addChild(closeButton);
            }

            if (!podButtonBar)
            {
                podButtonBar=new ButtonBar();
                podButtonBar.y=0;
                newGraphButton=new Button();
                newGraphButton.label="New Graph";
                newGraphButton.setStyle("icon", chartIcon16);
                newGraphButton.addEventListener(MouseEvent.CLICK, createNewGraph);

                saveButton=new Button();
                saveButton.label="Save as...";
                saveButton.setStyle("icon", saveIcon16);
                saveButton.addEventListener(MouseEvent.CLICK, saveHandler);

                podButtonBar.addChild(newGraphButton);
                //podButtonBar.addChild(visualButton);
                //podButtonBar.addChild(textualButton);
                podButtonBar.addChild(saveButton);
                this.addChild(podButtonBar);

                containerCanvas=new Canvas();

                xmlTextArea=new TextArea();
                xmlTextArea.text="xml content here";
                xmlTextArea.height=this.parent.height - 100;
                xmlTextArea.width=this.parent.width - 60;
                xmlTextArea.y=0;
                xmlTextArea.visible=false;
                xmlTextArea.editable=false;
                xmlTextArea.toolTip="This area is not editable for the moment";
                this.containerCanvas.addChild(xmlTextArea);

                tabbedEditor=new SuperTabNavigator();
                tabbedEditor.editableTabLabels=false;
                tabbedEditor.closePolicy=SuperTab.CLOSE_ALWAYS;
                //tabbedEditor.addEventListener(ChildExistenceChangedEvent.CHILD_REMOVE, closeTabHandler);
                tabbedEditor.y=0;
                tabbedEditor.visible=true;
                tabbedEditor.width=this.parent.width - 60;
                this.containerCanvas.addChild(tabbedEditor);

                visualButton=new Button();
                visualButton.label="Design";
                visualButton.setStyle("icon", visualIcon16);
                visualButton.y = this.height * 0.7 + 10;
                visualButton.addEventListener(MouseEvent.CLICK, switchToDesignView);

                textualButton=new Button();
                textualButton.label="XML View";
                textualButton.setStyle("icon", textualIcon16);
                textualButton.y = this.height * 0.7 + 10;
                textualButton.x = 85;
                textualButton.addEventListener(MouseEvent.CLICK, switchToXMLView);

                this.containerCanvas.addChild(visualButton);
                this.containerCanvas.addChild(textualButton);

                this.addChild(containerCanvas);
            }

            addEventListeners();
        }

        override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
        {
            super.updateDisplayList(unscaledWidth, unscaledHeight);

            // Shift the divider one pixel up if minimized so there isn't a gap between the left and right borders.
            // The bottom border is removed if minimized.
            var deltaY:Number=windowState == WINDOW_STATE_MINIMIZED ? -1 : 0;
            var graphics:Graphics=headerDivider.graphics;
            graphics.clear();
            graphics.lineStyle(1, getStyle("borderColor"));
            graphics.moveTo(1, titleBar.height + deltaY);
            graphics.lineTo(titleBar.width, titleBar.height + deltaY);

            controlsHolder.y=titleBar.y;
            controlsHolder.width=unscaledWidth;
            controlsHolder.height=titleBar.height;

            titleTextField.width=titleBar.width - getStyle("paddingLeft") - getStyle("paddingRight");
        }

        private function addEventListeners():void
        {
            titleBar.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDownTitleBar);
            titleBar.addEventListener(MouseEvent.DOUBLE_CLICK, onClickMaximizeRestoreButton);
            titleBar.addEventListener(MouseEvent.CLICK, onClickTitleBar);

            minimizeButton.addEventListener(MouseEvent.CLICK, onClickMinimizeButton);
            maximizeRestoreButton.addEventListener(MouseEvent.CLICK, onClickMaximizeRestoreButton);
            closeButton.addEventListener(MouseEvent.CLICK, onClickCloseButton);

            addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
        }

        private function onMouseDown(event:Event):void
        {
            // Moves the pod to the top of the z-index.
            parent.setChildIndex(this, parent.numChildren - 1);
        }

        private function onClickMinimizeButton(event:MouseEvent):void
        {
            dispatchEvent(new PodStateChangeEvent(PodStateChangeEvent.MINIMIZE));
            // Set the state after the event is dispatched so the old state is still available.
            minimize();
        }

        public function minimize():void
        {
            // Hide the bottom border if minimized otherwise the headerDivider and bottom border will be staggered.
            setStyle("borderSides", "left top right");
            windowState=WINDOW_STATE_MINIMIZED;
            height=MINIMIZED_HEIGHT;
            showControls=false;
        }

        private function onClickMaximizeRestoreButton(event:MouseEvent=null):void
        {
            showControls=true;
            if (windowState == WINDOW_STATE_DEFAULT)
            {
                dispatchEvent(new PodStateChangeEvent(PodStateChangeEvent.MAXIMIZE));
                // Call after the event is dispatched so the old state is still available.
                maximize();
            }
            else
            {
                dispatchEvent(new PodStateChangeEvent(PodStateChangeEvent.RESTORE));
                // Set the state after the event is dispatched so the old state is still available.
                windowState=WINDOW_STATE_DEFAULT;
                maximizeRestoreButton.selected=false;
            }
        }

        public function maximize():void
        {
            windowState=WINDOW_STATE_MAXIMIZED;

            _maximize=true;
            _maximizeChanged=true;
        }

        private function onClickCloseButton(event:MouseEvent):void
        {
            dispatchEvent(new PodStateChangeEvent(PodStateChangeEvent.CLOSE));
            // Set the state after the event is dispatched so the old state is still available.
            close();
        }

        public function close():void
        {
            //Alert.show("closing the pod");
            this.parent.removeChild(this);
        }

        private function onClickTitleBar(event:MouseEvent):void
        {
            if (windowState == WINDOW_STATE_MINIMIZED)
            {
                // Add the bottom border back in case we were minimized.
                setStyle("borderSides", "left top right bottom");
                onClickMaximizeRestoreButton();
            }
        }

        private function onMouseDownTitleBar(event:MouseEvent):void
        {
            if (windowState == WINDOW_STATE_DEFAULT) // Only allow dragging if we are in the default state.
            {
                dispatchEvent(new DragEvent(DragEvent.DRAG_START));
                dragStartX=x;
                dragStartY=y;
                dragStartMouseX=parent.mouseX;
                dragStartMouseY=parent.mouseY;
                dragMaxX=parent.width - width;
                dragMaxY=parent.height - height;

                // Use the stage so we get mouse events outside of the browser window.
                stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
                stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
            }
        }

        private function onMouseMove(e:MouseEvent):void
        {
            // Make sure we don't go off the screen on the right.
            var targetX:Number=Math.min(dragMaxX, dragStartX + (parent.mouseX - dragStartMouseX));
            // Make sure we don't go off the screen on the left.
            x=Math.max(0, targetX);

            // Make sure we don't go off the screen on the bottom.
            var targetY:Number=Math.min(dragMaxY, dragStartY + (parent.mouseY - dragStartMouseY));
            // Make sure we don't go off the screen on the top.
            y=Math.max(0, targetY);
        }

        private function onMouseUp(event:MouseEvent):void
        {
            dispatchEvent(new DragEvent(DragEvent.DRAG_COMPLETE));

            stage.removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
            stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
        }

        public function set showControls(value:Boolean):void
        {
            _showControls=value;
            _showControlsChanged=true;
            invalidateProperties();
        }

        override protected function commitProperties():void
        {
            super.commitProperties();

            if (_showControlsChanged)
            {
                controlsHolder.visible=_showControls;
                _showControlsChanged=false;
            }

            if (_maximizeChanged)
            {
                maximizeRestoreButton.selected=_maximize;
                _maximizeChanged=false;
            }
        }

        private function resizeHandler(event:ResizeEvent):void
        {
            this.tabbedEditor.width=this.width * 0.9;
            this.xmlTextArea.width=this.width * 0.9;
            this.xmlTextArea.height=this.height * 0.7;
            if (xmlTextArea.height >= this.tabbedEditor.height) {
                textualButton.y = this.height * 0.7 + 10;
                visualButton.y = this.height * 0.7 + 10;
            }
            else {
                textualButton.y = this.tabbedEditor.height + 10;
                visualButton.y = this.tabbedEditor.height + 10;
            }
            
        }

        ///////////////////////////////////////////////////////////////////
        // functions linked with the pod
        ///////////////////////////////////////////////////////////////////
        private function createNewGraph(event:MouseEvent):void
        {
            var graphArrayCollection:ArrayCollection=new ArrayCollection();
            var graphXML:GraphXML=new GraphXML("New Graph", this.graphAvailable, graphArrayCollection, "lineChartConst");
            tabbedEditor.addChild(graphXML);
        }

        private function switchToDesignView(event:MouseEvent):void
        {
            xmlTextArea.visible=false;
            tabbedEditor.visible=true;
        }

        private function switchToXMLView(event:MouseEvent):void
        {
            xmlTextArea.visible=true;
            tabbedEditor.visible=false;
            xmlTextArea.text=designToXML().toXMLString();
        }

        private function saveHandler(event:MouseEvent):void
        {
            exportRequest=new URLRequest();

            var readyToDownloadMessage:Alert;

            //var data : String = xmlTextArea.text;
            var data:String=designToXML().toXMLString();

            downloadFileRef=new FileReference();

            // Build the request data
            var requestData:URLVariables=new URLVariables("xmldata=" + data);

            // Setup the request
            exportRequest.data=requestData;
            exportRequest.url=XML_DOWNLOAD_END_POINT;
            exportRequest.method=URLRequestMethod.POST;

            // Ask the user for its permission to download the file
            readyToDownloadMessage=Alert.show('Do you want to save this configuration file to your hard drive ?', 'The file is ready to be downloaded', (Alert.YES | Alert.NO), null, readyToDownloadMessageResponseHandler, null, Alert.OK);
        }

        /**
         * Handles the user response to the "ready to download" message
         */
        private function readyToDownloadMessageResponseHandler(event:CloseEvent):void
        {
            if (event.detail == Alert.YES)
            {
                downloadFileRef.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
                downloadFileRef.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
                downloadFileRef.download(exportRequest, this.title);
            }
        }

        /**
         * Handles export security error events
         */
        private function securityErrorHandler(event:SecurityErrorEvent):void
        {

            // Display a message
            Alert.show("securityErrorHandler: " + event.toString());
        }

        /**
         * Handles export I/O error events
         */
        private function ioErrorHandler(event:IOErrorEvent):void
        {
            // Display a message
            Alert.show("ioErrorHandler: " + event.toString());
        }

        public function designToXML():XML
        {
            var seriesDef:ArrayCollection=new ArrayCollection();
            var doc:XML=new XML();
            doc=<monitor absciss="time" separator=";" timeformat="long" ></monitor>;
            for (var i:int=0; i < this.tabbedEditor.getChildren().length; i++)
            {
                //the graph
                var graphDefNode:XMLList=XMLList("<graph id='g" + i + "'></graph>");
                var title:XMLList=XMLList("<title>" + ((this.tabbedEditor.getChildAt(i)) as GraphXML).getGraphTitle() + "</title>");
                graphDefNode.appendChild(title);
                var chartType:XMLList=XMLList("<chartType>" + ((this.tabbedEditor.getChildAt(i)) as GraphXML).getGraphType() + "</chartType>");
                graphDefNode.appendChild(chartType);

                //the series of the current graph
                seriesDef=((this.tabbedEditor.getChildAt(i)) as GraphXML).getDataGridContent();
                for each (var element:Object in seriesDef)
                {
                    var seriesDefNode:XMLList=XMLList("<serie-def id='" + element.Id + "'></serie-def>");
                    if (element.Name != "" && element.Name != null)
                    {
                        var name:XMLList=XMLList("<name>" + element.Name + "</name>");
                        seriesDefNode.appendChild(name);
                    }
                    if (element.ServerSource != "" && element.ServerSource != null)
                    {
                        var server:XMLList=XMLList("<server>" + element.ServerSource + "</server>");
                        seriesDefNode.appendChild(server);
                    }
                    if (element.PatternFilter != "" && element.PatternFilter != null)
                    {
                        var pattern:XMLList=XMLList("<pattern>" + element.PatternFilter+ "</pattern>");
                        seriesDefNode.appendChild(pattern);
                    }
                    if (element.Description != "" && element.Description != null)
                    {
                        var description:XMLList=XMLList("<description>" + element.Description + "</description>");
                        seriesDefNode.appendChild(description);
                    }
                    var y:XMLList=XMLList("<y col='" + element.IndicatorName + "' type='double' />");
                    seriesDefNode.appendChild(y);

                    //append the reference of the serie to the correct graph
                    var serieRef:XMLList=XMLList("<serie ref='" + element.Id + "' />");
                    graphDefNode.appendChild(serieRef);

                    doc.appendChild(seriesDefNode);
                }
                doc.appendChild(graphDefNode);
            }
            return doc;
        }

        public function xmlToDesign(doc:XML):ArrayCollection
        {
            var graphs:XMLList=doc.child("graph");
            var serieDefs:XMLList=doc.child("serie-def");
            var graphsList:ArrayCollection=new ArrayCollection();
            for each (var node:XML in graphs)
            {
                var graphArrayCollection:ArrayCollection=new ArrayCollection();
                var id:int=0;
                for each (var serieNode:XML in node.child("serie"))
                {
                    for each (var nodeGraph:XML in serieDefs)
                    {
                        if (nodeGraph.@id == serieNode.@ref)
                        {
                            var serieId:String="";
                            if (nodeGraph.@id.toString().length > 0)
                            {
                                serieId=nodeGraph.@id;
                            }
                            var value:String="";
                            if (nodeGraph.y.@col.toString().length > 0)
                            {
                                value=nodeGraph.y.@col;
                            }
                            var name:String="";
                            if (nodeGraph.name.toString().length > 0)
                            {
                                name=nodeGraph.name;
                            }
                            var pattern:String="";
                            if (nodeGraph.pattern.toString().length > 0)
                            {
                                pattern=nodeGraph.pattern;
                            }
                            var server:String="";
                            if (nodeGraph.server.toString().length > 0)
                            {
                                server=nodeGraph.server;
                            }
                            var description:String="";
                            if (nodeGraph.description.toString().length > 0)
                            {
                                description=nodeGraph.description;
                            }
                            graphArrayCollection.addItem({Id: serieId, IndicatorName: value, Name: name, PatternFilter: pattern, ServerSource: server, Description: description,
                                //color:"0x0000FF", 
                                    Option: id});
                            id++;
                        }
                    }
                }
                var graphXML:GraphXML=new GraphXML(node.title, this.graphAvailable, graphArrayCollection, node.chartType);
                graphsList.addItem(graphXML);
            }
            return graphsList;
        }

        /**
         * set the data in the tabs and the text area
         */
        public function setGraphs(graphList:ArrayCollection, xmlString:String):void
        {
            for (var i:int=0; i < graphList.length; i++)
            {
                tabbedEditor.addChild((graphList.getItemAt(i)) as DisplayObject);
            }
            xmlTextArea.text=xmlString;
            xmlTextArea.percentHeight=this.parent.height * 0.9;
            xmlTextArea.percentWidth=this.parent.width * 0.9;
        }

        public function closeTabHandler(event:ChildExistenceChangedEvent):void
        {
            Alert.show('Are you sure to delete it ?', 'This graph is ready to be deleted', (Alert.YES | Alert.NO), null, closeResponseHandler, null, Alert.OK);
        }

        public function closeResponseHandler(event:CloseEvent):void
        {
            if (event.detail == Alert.YES)
            {
                EoSLogger.info("removed graph");
            }
        }
    }
}
